<?php

namespace EscapeWork\Manager\Console;

use Illuminate\Support\Str;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\File;
use Illuminate\Console\AppNamespaceDetectorTrait;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputArgument;

class CrudGeneratorTemplateCommand extends Command
{

    use AppNamespaceDetectorTrait;

    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'manager:crud:template {template?} {--with-migration}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Cria um CRUD com base em um template pré definido (Opções: news)';

    /**
     * @var  mixed
     */
    protected $key, $namespace;

    /**
     * Execute the console command.
     *
     * @return void
     */
    public function fire()
    {
        $templates = require(__DIR__ . '/../../config/templates.php');

        if (! $this->argument('template') || ! array_key_exists($this->argument('template'), $templates)) {
            $this->showOptions($templates);
            return;
        }

        $this->key       = $this->argument('template');
        $template        = $templates[$this->key];
        $this->namespace = $this->getAppNamespace();

        $this->info(' -> Gerando os arquivos para o ' . $template['description']);
        $this->line('');

        foreach ($template['files'] as $file => $data) {
            $path = base_path() . '/' . $data['path'];

            # se for pasta, copia o diretório inteiro
            if ($data['extension'] == 'folder') {
                $this->copyFolder($path, $data);
                continue;
            }

            # se não, cria os arquivos
            $this->createFile($path, isset($data['name']) ? $data['name'] : $file, $data);
        }

        $this->line('');
        $this->info(' -> Cadastro criado com sucesso!');
        $this->line('');

        # rotas
        $this->appendRoutes($template);

        # menu
        $this->addToMenu($template);

        if ($this->option('with-migration')) {
            $this->info(' -> E não esqueça de executar a migration!');
        }

        if (isset($template['obs']) && $template['obs'] !== '') {
            $this->comment(' -> ' . $template['obs']);
        }
    }

    public function copyFolder($path, $data)
    {
        File::copyDirectory(
            __DIR__ . '/../../resources/stubs/'.$this->argument('template').'/' . $data['stub'], # diretório original
            $path
        );

        $this->comment(' -> Diretório ' . $data['path'] . ' criado!');
    }

    public function createFile($path, $file, $data)
    {
        $stub = File::get(__DIR__ . '/../../resources/stubs/'.$this->argument('template').'/' . $data['stub']);
        $stub = str_replace('{{namespace}}', $this->namespace, $stub);
        $file = str_replace('{{datetime}}', date('Y_m_d_His'), $file);

        # criando o diretório se não existe
        if (! File::isDirectory($path)) {
            File::makeDirectory($path);
        }

        if (! File::exists($path . '/' . $file . $data['extension'])) {
            File::put($path . '/' . $file . $data['extension'], $stub);

            $this->comment(' -> ' . $data['path'] . '/' . $file . $data['extension'] . ' criado com sucesso!');
        } else {
            $this->error(' -> ' . $data['path'] . '/' . $file . $data['extension'] . ' já existe e não foi sobreescrito.');
        }
    }

    public function showOptions($templates)
    {
        $this->info('Os templates disponíveis são os seguintes: ');
        $this->line('');

        $headers = ['Template', 'Descrição', 'Uso'];
        $content = [];

        foreach ($templates as $type => $template) {
            $content[] = [
                $type, $template['description'], 'php artisan manager:crud:template ' . $type
            ];
        }

        $this->table($headers, $content);
    }

    protected function appendRoutes($template)
    {
        $routes = '';

        if (is_array($template['route'])) {
            $routes .= "Route::get('".$template['route']['url']."', ['as' => '".$template['route']['name'].".index', 'uses' => '".$template['route']['controller']."@index']);";
            $routes .= "Route::get('".$template['route']['url']."/inserir', ['as' => '".$template['route']['name'].".create', 'uses' => '".$template['route']['controller']."@create']);";
            $routes .= "Route::post('".$template['route']['url']."', ['as' => '".$template['route']['name'].".store', 'uses' => '".$template['route']['controller']."@store']);";
            $routes .= "Route::get('".$template['route']['url']."/{id}/editar', ['as' => '".$template['route']['name'].".edit', 'uses' => '".$template['route']['controller']."@edit']);";
            $routes .= "Route::put('".$template['route']['url']."/{id}', ['as' => '".$template['route']['name'].".update', 'uses' => '".$template['route']['controller']."@update']);";
            $routes .= "Route::put('".$template['route']['url']."/{id}/basic', ['as' => '".$template['route']['name'].".updateBasic', 'uses' => '".$template['route']['controller']."@updateBasic']);";
            $routes .= "Route::delete('".$template['route']['url']."/{id}', ['as' => '".$template['route']['name'].".destroy', 'uses' => '".$template['route']['controller']."@destroy']);";
        } else {
            $routes = File::get(__DIR__ . '/../../resources/stubs/'.$this->argument('template').'/' . $template['route']);
        }

        # adicionando as rotas no arquivo
        File::append(base_path('routes/manager.php'), $routes);
        $this->info(' -> Rotas adicionadas com sucesso!');
    }

    protected function addToMenu($template)
    {
        if (! is_array($template['route']) && ! isset($template['menu'])) {
            return;
        }

        # adicionando no menu
        if (! isset($template['menu'])) {
            $html = '<li class="{{ $page == \''.$template['route']['name'].'\' ? \'active\' : null }}">
        <a href="{{ route(\'manager.'.$template['route']['name'].'.index\') }}">' . $template['title'] . '</a>
    </li>';
        } else {
            $html = File::get(__DIR__ . '/../../resources/stubs/'.$this->argument('template').'/' . $template['menu']);
        }

        if (File::exists($menu = base_path('resources/views/manager/menu.blade.php'))) {
            File::append($menu, "\n" . $html);

            $this->line('');
            $this->info(' -> Adicionamos o HTML com o link na view do menu (resources/views/manager/menu.blade.php)');
        } else {
            $this->line('');
            $this->comment(' -> E adicione no seu menu');
            $this->line($html);
        }
    }
}
